"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const Effects = __importStar(require("./effects"));
describe("none()", () => {
    test("should be equal to []", () => {
        expect(Effects.none()).toEqual([]);
    });
});
describe("delay()", () => {
    test("should return a array containing a single function", () => {
        const effects = Effects.delay("FROG", 100);
        expect(effects[0]).toBeInstanceOf(Function);
        expect(effects).toHaveLength(1);
    });
    test("should call setTimeout with the specified time", () => {
        jest.useFakeTimers();
        const delay = 100;
        const action = "FROG";
        const delayEffect = Effects.delay(action, delay)[0];
        const mockDispatch = jest.fn();
        delayEffect(mockDispatch);
        expect(setTimeout).toHaveBeenCalledTimes(1);
        expect(setTimeout).toHaveBeenCalledWith(expect.any(Function), delay);
        jest.runAllTimers();
        expect(mockDispatch).toHaveBeenCalledTimes(1);
        expect(mockDispatch).toHaveBeenCalledWith(action);
    });
});
describe("action()", () => {
    test("should return a array containing a single function", () => {
        const effects = Effects.action("FROG");
        expect(effects[0]).toBeInstanceOf(Function);
        expect(effects).toHaveLength(1);
    });
    test("should call dispatch with the specified action", () => {
        jest.useFakeTimers();
        const action = "SHOEHORN";
        const actionEffect = Effects.action(action)[0];
        const mockDispatch = jest.fn();
        actionEffect(mockDispatch);
        expect(mockDispatch).toHaveBeenCalledTimes(1);
        expect(mockDispatch).toHaveBeenCalledWith(action);
    });
});
describe("fromPromise()", () => {
    test("should return a array containing a single function", () => {
        const effects = Effects.fromPromise(() => __awaiter(this, void 0, void 0, function* () { return Promise.resolve("FROG"); }));
        expect(effects[0]).toBeInstanceOf(Function);
        expect(effects).toHaveLength(1);
    });
    test("should invoke provided dispatch once promise has resolved", () => __awaiter(this, void 0, void 0, function* () {
        const action = "BOOTS & CATS";
        const effects = Effects.fromPromise(() => Promise.resolve(action));
        const mockDispatch = jest.fn();
        yield effects[0](mockDispatch);
        expect(mockDispatch).toHaveBeenCalledTimes(1);
    }));
});
describe("fromFunction()", () => {
    test("should return input function in array containing single item", () => {
        const mockFn = jest.fn();
        const effects = Effects.fromFunction(mockFn);
        expect(effects[0]).toBe(mockFn);
        expect(effects).toHaveLength(1);
    });
});
describe("fromIterator()", () => {
    test("should return a array containing a single function", () => {
        const effects = Effects.fromIterator([1, 2, 3]);
        expect(effects[0]).toBeInstanceOf(Function);
        expect(effects).toHaveLength(1);
    });
    test("should dispatch each item in iterable array when invoked", () => {
        const effects = Effects.fromIterator([1, 2, 3]);
        const mockDispatch = jest.fn();
        effects[0](mockDispatch);
        expect(mockDispatch).toHaveBeenCalledTimes(3);
        expect(mockDispatch).toHaveBeenNthCalledWith(1, 1);
        expect(mockDispatch).toHaveBeenNthCalledWith(2, 2);
        expect(mockDispatch).toHaveBeenNthCalledWith(3, 3);
    });
});
describe("combine()", () => {
    test("should concatenate two effects together", () => {
        const effect1 = Effects.fromIterator([1, 2, 3]);
        const effect2 = Effects.action(4);
        const effects = Effects.combine(effect1, effect2);
        expect(effects[0]).toBeInstanceOf(Function);
        expect(effects[1]).toBeInstanceOf(Function);
        expect(effects).toHaveLength(2);
    });
});
